import numpy as np
import datetime
from matplotlib import pyplot as plt
from sarlab.tools.conversion  import geo_to_utm

def _transect_positions(start_pos, end_pos, distance):
    start_e,start_n,_ = geo_to_utm(np.asarray([start_pos[0]]), np.asarray([start_pos[1]]))
    end_e,end_n,_ = geo_to_utm(np.asarray([end_pos[0]]), np.asarray([end_pos[1]]))
    del_utm = np.asarray((end_e[0] - start_e[0], end_n[0] - start_n[0]))
    end_dist = np.sqrt(del_utm[0] ** 2 + del_utm[1] ** 2)
    del_pos = end_pos - start_pos
    pos = np.zeros((len(distance), 2))
    for ii in np.arange(len(distance)):
        pos[ii, :] = start_pos + del_pos * distance[ii] / end_dist
    return pos

class Snow_survey_transect_2018():
    def __init__(self, date, name, distance, depth, swe, start_pos, end_pos):
        self.date = datetime.datetime.strptime(date, '%Y%m%d')
        self.name = name
        self.distance = np.asarray(distance, dtype=np.float)
        self.depth = np.asarray(depth, dtype=np.float)*0.01 #cm to m
        self.swe = np.asarray(swe, dtype=float)*0.01#cm to m
        self.swe.astype(float)
        self.swe[np.asarray(swe) == -9999] = np.nan
        self.start_pos = np.asarray(start_pos)
        self.end_pos = np.asarray(end_pos)
        self.position = _transect_positions(self.start_pos, self.end_pos, self.distance)

    def plot(self):
        plt.plot(self.distance, self.depth, 'b')
        plt.ylim((0.,150.))
        plt.title('Transect: ' + self.name)
        plt.xlabel('Distance [m]')
        plt.ylabel('Snow depth [cm]')
        ax1 = plt.gca()
        ax2 = ax1.twinx()
        ax2.set_ylabel('SWE [cm]')
        swe_plot = self.swe[np.isfinite(self.swe)]
        distance_swe = self.distance[np.isfinite(self.swe)]
        ax2.plot(distance_swe, swe_plot, 'r')
        plt.ylim((0., 25.))

    def plot_pos(self):
        plt.plot(self.position[:,0], self.position[:,1])

t_sa_2018 = Snow_survey_transect_2018(
    '20180315', 'SA',
    [0,10,20,30,40,50,60,70,80,90,100,110,120,130,140,150,160,170,180,190,200],
    [48,55,50,50,50,73,54,60,72,90,51,66,53,52,51,52,59,55,61,69,54],
    [7,12,7,8.5,8.5,8.5,11.5,8.5,15,14.5,10.5,12.5,8,9.5,7,11.5,8.5,9.5,11.5,11.5,3.5],
    [-133.772313,68.397809],
    [-133.777663,68.397921])
t_sb_2018 = Snow_survey_transect_2018(
    '20180315', 'SB',
    (0,10,20,30,40,50,60,70,80,90,100,110,120,130,140),
    (70,51,69,60,63,58,40,46,67,55,55,68,53,50,51),
    (14.5,8.5,12.5,11.5,12.5,11.5,10.5,9.5,9.5,10.5,11,12,9.5,7.5,9.5),
    (-133.734163,68.375799),
    (-133.732258,68.374878))
t_sc_2018 = Snow_survey_transect_2018(
    '20180315', 'SC',
    (0,10,20,30,40,50,60,70,80,90,100),
    (47,55,59,44,59,55,51,42,52,43,59),
    (8,14,9.5,10,11,11.5,12.5,14.5,8.5,4.5,9.5),
    (-133.740908,68.380290),
    (-133.738534,68.380559))

